#include <stdio.h>
#include <string.h>
#include "forms.h"
#include "avrform.h"
#include "hwt.h"
#include "saveform.h"
#include "fusesmega.h"
#include "common.h"
#include "lowio.h"
#include "bbang.h"
#include "operation.h"
#include "id.h"
#include "dumphex.h"

extern chip_id *chip,idt[];
extern int nbytes;
extern unsigned char membuf[];
extern FD_saveform *fd_saveform;
extern FD_avrform *fd_avrform;
extern FD_hwt *fd_hwt;
extern FD_fusesmega *fd_fusesmega;

/*** callbacks and freeobj handles for form saveform ***/
const char * const name[3]={"Motorola SREC","Intel HEX","Binary"};
int fmt=0;
int save_done;
int fusesmega_done;

void format_menu(FL_OBJECT *ob, long data)
{
  fmt=fl_get_menu(ob)-1;
  fl_set_object_label(fd_saveform->format_name,name[fmt]);
}

void save_file(FL_OBJECT *ob, long data)
{
    char *fn,buf[256];
    FILE *fp;

    save_done=1;
    if (data==0) return;
    fn=(char *)fl_get_input(fd_saveform->filename);
    if ((fp=fopen(fn,"w"))==NULL) {
        message("***** Error while opening save file",1);
        return;
    }
    switch(fmt) {
    case 0:   dumpsrec(membuf,nbytes,0,fp); break;
    case 1:   dumpihx(membuf,nbytes,0,fp); break;
    case 2:   fwrite(membuf,1,nbytes,fp); break;
    }
    fclose(fp);
    sprintf(buf,"> Buffer sucessfully saved on \"%s\" (Format: %s)",fn,name[fmt]);
    message(buf,4);
}

void browse(FL_OBJECT *ob, long data)
{
    char *fn,*p,*p1;
    fn=(char *)fl_show_fselector("Select File","./","*.[sib][rhi][exn]*","");
    if (!fn) return;
    p=fn; while (*p) p++;
    p1=p;
    while (*p!='/') p--;
    while (*p1!='.' && p1>p) p1--;
    if (strcmp(p1,".srec")==0 || strcmp(p1,".SREC")==0 ) {
        fmt=0;
    } else
    if (strcmp(p1,".ihx")==0 || strcmp(p1,".IHX")==0 || strcmp(p1,".HEX")==0){
        fmt=1;
    } else {
        fmt=2;
    }
    fl_set_object_label(fd_saveform->format_name,name[fmt]);
    fl_set_input(fd_saveform->filename,++p);
}

//-------------------------------------------------------------------------
/*** callbacks and freeobj handles for form avrform ***/
//-------------------------------------------------------------------------

int loadhex(char *);

void butload(FL_OBJECT *ob, long data)
{
  char *fn;
  int i;
  FILE *fp;
  char buf[256];

  fn=(char *)fl_show_fselector("Select File","./","*.[sib][rhi][exn]*","");
  if (!fn) return;

  if (loadhex(fn)) {
       if ((fp=fopen(fn,"r"))==NULL) {
           message("***** Error opening file",1);
           return;
       }
       nbytes=fread(membuf,1,MAXBUF,fp);
       if (nbytes==MAXBUF) {
           if (fread(buf,1,1,fp)==1)
               message("<> Warning: File do not fit in buffer.",2);
       }
       fclose(fp);
       sprintf(buf,"----- File Loaded :%s",fn);
       message(buf,4);
       sprintf(buf,"> Format: Binary, Base:0x0, Size :%d",nbytes);
       message(buf,0);
  };
  if (nbytes>chip->flash_size)
      message("<> Warning: Data will not fit in Program Flash",2);
  if (nbytes>chip->eeprom_size)
      message("<> Warning: Data will not fit in EEPROM",2);
  dump_mem();
}

void butsave(FL_OBJECT *ob, long data)
{
    FILE *fp;
    char buf[256];

    save_done=0;
    fl_show_form(fd_saveform->saveform,FL_PLACE_CENTERFREE,FL_FULLBORDER,"SAVE");
    fl_deactivate_form(fd_avrform->avrform);
    fl_set_object_label(fd_saveform->format_name,name[fmt]);

    while(!save_done) {usleep(20000); fl_check_forms();}

    fl_hide_form(fd_saveform->saveform);
    fl_activate_form(fd_avrform->avrform);
}

void butid(FL_OBJECT *ob, long data)
{
    if (SPI_sync()) return;
    identify_device();
    set_Vcc_L();
}

void butprg(FL_OBJECT *ob, long data)
{
  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  if (fl_show_question("FLASH PROGRAMMING.\nDo you want to continue?", 0)==0) return;
  message("----- FLASH PROGRAMMING started",4);
  if (SPI_sync()) return;
  if (!write_flash()) message("----- FLASH PROGRAMMING OK",4);;
  set_Vcc_L();
}

void buterase(FL_OBJECT *ob, long data)
{

  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  if (fl_show_question("CHIP ERASE.\nDo you want to continue?", 0)==0) return;
  message("----- CHIP ERASE started",4);
  if (SPI_sync()) return;
  chip_erase();
  message("----- FLASH READING started",4);
  if (SPI_sync()) return;
  if (check_flash()) message("***** BLANK CHECK Failed",1);
  else message("----- BLANK CHECK OK. CHIP ERASE OK.",4);
  set_Vcc_L();
}

void butrd(FL_OBJECT *ob, long data)
{
  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  message("----- FLASH READING started",4);
  if (SPI_sync()) return;
  if (!read_flash()) message("----- FLASH READING OK",4);
  dump_mem();
  set_Vcc_L();
}

void butrdee(FL_OBJECT *ob, long data)
{
  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  message("----- EEPROM READING started",4);
  if (SPI_sync()) return;
  if(!read_eeprom()) message("----- EEPROM READING OK",4);
  dump_mem();
  set_Vcc_L();
}

void butwree(FL_OBJECT *ob, long data)
{
  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  if (fl_show_question("EEPROM WRITE.\nDo you want to continue?", 0)==0) return;
  message("----- EEPROM WRITE started",4);
  if (SPI_sync()) return;
  if (!write_eeprom())message("----- EEPROM WRITE OK",4);
  set_Vcc_L();
}

void butblank(FL_OBJECT *ob, long data)
{
  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  message("----- BLANK CHECK (Flash READ) started",4);
  if (SPI_sync()) return;
  if (!check_flash()) message("----- BLANK CHECK OK",4);
  else message("***** BLANK CHECK failed. (Chip Erase is required)",1);
  set_Vcc_L();
}

void butverify(FL_OBJECT *ob, long data)
{
  if (!chip->flash_size) {
      message("***** You must select a device",1);
      return;
  }
  message("----- FLASH VERIFY (Flash READ) started",4);
  if (SPI_sync()) return;
  if (!verify_flash())
      message("----- Buffer and chip's Flash match. Verify OK",4);
  else message("***** Verify failed.",1);
  set_Vcc_L();
}

void exitb(FL_OBJECT *ob, long data)
{
  exit(0);
}

void bufbrow(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void mesbrow(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void deviceselect(FL_OBJECT *ob, long data)
{
  int i;
  i=fl_get_menu(ob);
  chip=&idt[i];
  show_chip();
}



int hwt_done;

void hwtst(FL_OBJECT *ob, long data)
{
  hwt_done=0;
  set_Vcc_L();
  set_SCK_L();
  set_MOSI_L();
  fl_set_button(fd_hwt->vddl,1);
  fl_set_button(fd_hwt->vddh,0);
  fl_set_button(fd_hwt->sckl,1);
  fl_set_button(fd_hwt->sckh,0);
  fl_set_button(fd_hwt->mosil,1);
  fl_set_button(fd_hwt->mosih,0);
  fl_show_form(fd_hwt->hwt,FL_PLACE_CENTERFREE,FL_FULLBORDER,"Hardware Test Tool");
  fl_deactivate_form(fd_avrform->avrform);
  do {
        fl_check_forms();
        if (get_MISO()) {
            fl_set_button(fd_hwt->misol,0);
            fl_set_button(fd_hwt->misoh,1);
        } else {
            fl_set_button(fd_hwt->misol,1);
            fl_set_button(fd_hwt->misoh,0);
        }
        usleep(50000);
  } while (hwt_done==0);
  fl_hide_form(fd_hwt->hwt);
  fl_activate_form(fd_avrform->avrform);
  set_Vcc_L();
  set_SCK_L();
  set_MOSI_L();
}

void butfuses(FL_OBJECT *ob, long data)
{
  fusesmega_done=0;
  fl_show_form(fd_fusesmega->fusesmega,FL_PLACE_CENTERFREE,FL_FULLBORDER,"FUSE PROGRAMMING");
  fl_deactivate_form(fd_avrform->avrform);
  
  if (!(chip->fuses&4)) { 
     fl_hide_object(fd_fusesmega->fusesEX); // No extended fuses
  } else {
     fl_show_object(fd_fusesmega->fusesEX);
  }
  do {
        fl_check_forms();
        if (get_MISO()) {
            fl_set_button(fd_hwt->misol,0);
            fl_set_button(fd_hwt->misoh,1);
        } else {
            fl_set_button(fd_hwt->misol,1);
            fl_set_button(fd_hwt->misoh,0);
        }
        usleep(50000);
  } while (fusesmega_done==0);
  fl_hide_form(fd_fusesmega->fusesmega);
  fl_activate_form(fd_avrform->avrform);
}
