#include <stdlib.h>
#include <stdio.h>
#include "common.h"
#include "delay.h"
#include "lowio.h"
#include "bbang.h"
#include "operation.h"
#include "id.h"

#include "forms.h"
#include "avrform.h"
#include "hwt.h"
#include "saveform.h"
#include "fusesmega.h"

#ifndef PROG_DEV
#define PROG_DEV "/dev/ttyS0"
#endif

char *prog_dev=PROG_DEV;
extern chip_id idt[];
chip_id *chip=idt;
int nbytes=0;
unsigned char membuf[MAXBUF];

FD_avrform *fd_avrform;
FD_hwt *fd_hwt;
FD_saveform *fd_saveform;
FD_fusesmega *fd_fusesmega;

void message(char *msg,int color)
{
    int i;
    char buf[256];
    if (color) {
        sprintf(buf,"@C%d%s",color,msg);
        fl_add_browser_line(fd_avrform->mesbrow,buf);
    } else fl_add_browser_line(fd_avrform->mesbrow,msg);
    i=fl_get_browser_maxline(fd_avrform->mesbrow);
    if (i>5) fl_set_browser_topline(fd_avrform->mesbrow,i-5);
}


void dump_mem()
{
    int i,j,addr;
    char buf[256];
    fl_clear_browser(fd_avrform->bufbrow);
    for (addr=0;addr<nbytes;addr+=16) {
        sprintf(buf,"@C4 %04X  ",addr);
        for(i=0;i<16;i++)
            if (addr+i<nbytes) sprintf(&buf[10+3*i],"%02X ",membuf[addr+i]);
            else sprintf(&buf[10+3*i],"   ");
        buf[10+3*16]=' ';
        j=nbytes-addr; if (j>16) j=16;
        for(i=0;i<j;i++)
            buf[10+3*16+1+i]=isprint(membuf[addr+i])?membuf[addr+i]:'.';
        buf[10+3*16+1+i]=0;
        fl_add_browser_line(fd_avrform->bufbrow,buf);
    }
}

void show_chip()
{
    char buf[256];
    fl_set_object_label(fd_avrform->devicename,chip->name);
    sprintf(buf,"%d",chip->flash_size);
    fl_set_object_label(fd_avrform->deviceflash,buf);
    sprintf(buf,"%d",chip->eeprom_size);
    fl_set_object_label(fd_avrform->deviceeeprom,buf);
    if (chip->fuses) {
      fl_show_object(fd_avrform->fuses);
      fl_activate_object(fd_avrform->fuses);
    }else {
      fl_deactivate_object(fd_avrform->fuses);
      fl_hide_object(fd_avrform->fuses);
    }
}

void refresh_percent(int data)
{
   fl_set_slider_value(fd_avrform->slpercent,data);
   fl_check_forms();
}

int main(int argc, char *argv[])
{
   int i;
   char *p,buf[2048];

   // Parse options

   for (i=1;i<argc;i++) {
       if (argv[i][0]=='-') {
           switch(argv[i][1]) {
           case 'h':  printf("Usage %s [options]\n",argv[0]);
                      printf("Options  :\n");
                      printf("-h       : Print this help\n");
                      printf("-d <dev> : Use the <dev> device for programmer\n");
                      break;
           case 'd':  prog_dev=argv[++i];
                      break;
           }
       }
   }

   fl_initialize(&argc, argv, 0, 0, 0);
   fd_avrform = create_form_avrform();
   fd_hwt = create_form_hwt();
   fd_saveform = create_form_saveform();
   fd_fusesmega = create_form_fusesmega();

   /* fill-in form initialization code */

   fl_set_browser_fontstyle(fd_avrform->mesbrow,FL_FIXED_STYLE);
   fl_set_browser_fontsize(fd_avrform->mesbrow,14);
   fl_set_browser_fontstyle(fd_avrform->bufbrow,FL_FIXED_STYLE);
   fl_set_browser_fontsize(fd_avrform->bufbrow,14);
   fl_deactivate_object(fd_avrform->slpercent);
   fl_deactivate_object(fd_avrform->fuses);
   fl_hide_object(fd_avrform->fuses);
   fl_deactivate_object(fd_hwt->misol);
   fl_deactivate_object(fd_hwt->misoh);

   for (i=1,p=buf;idt[i].id;i++) {
       sprintf(p,"%s|",idt[i].name);
       while (*p) p++;
   }
   fl_set_menu(fd_avrform->deviceselect,buf);
   fl_set_menu(fd_saveform->format_menu,"SREC|IHX|Binary");
   delay_calibrate(); 
   init_io();
   show_chip();
   /* show the first form */
   fl_show_form(fd_avrform->avrform,FL_PLACE_CENTERFREE,FL_FULLBORDER,
   	        "GPL AVR Programmer by Jess Arias");
   fl_do_forms();
   return 0;
}
