#include <stdlib.h>
#include <stdio.h>
#include "lowio.h"
#include "common.h"
#include "delay.h"

unsigned char spibyte(unsigned char byte)
{
    int i;
    unsigned char wby=byte,rby=0;

    for(i=0;i<8;i++) {
        if (wby&0x80) set_MOSI_H(); else set_MOSI_L();
        wby<<=1;
        udelay(TCKH);
        set_SCK_H();
        udelay(TCKH);
        rby<<=1;
        if (get_MISO()) rby++;
        set_SCK_L();
    }
    return rby;
}

int SPI_sync()
{
    int retry,res;
    char buf[256];

    //message("----- Powering-up/Resetting AVR MCU...",4);
    set_Vcc_L();    // Power-OFF
    set_SCK_L();
    set_MOSI_L();
    usleep(500000); // Be sure there is no power
    set_Vcc_H();    // Power-ON
    //message("----- Establishing SPI communication with AVR MCU...",4);
    usleep(100000); // Enough time to power-up and oscillator start-up

    for (retry=0;retry<32;retry++) {
        spibyte(0xac);  //Programming Enable
        spibyte(0x53);
        res=spibyte(0);
        spibyte(0);
        if (res==0x53) break; // OK, ready for programming
        // No sync. Insert a clock pulse
        udelay(TCKH/2);
        set_SCK_H();
        udelay(TCKH);
        set_SCK_L();
        udelay(TCKH);
    }
    if (retry==32) {
        message("***** Error: No Sync",1);
        set_Vcc_L();    // Power-OFF
        return 1;
    }
    if (retry) {
        sprintf(buf,"<> Notice: Sync after %d trials",retry);
        message(buf,2);
    }
    //sprintf(buf,"> OK. AVR MCU is alive"); message(buf,0);
    return 0;
}

//********************* MCU programming commands ***********************

unsigned char cmd_read_signature(int address)
{
        spibyte(0x30);
        spibyte(0);
        spibyte(address&3);
        return spibyte(0);
}

unsigned char cmd_read_program(int address)
{
        spibyte((address&1)?0x28:0x20);
        spibyte((address>>9)&0xff);
        spibyte((address>>1)&0xff);
        return spibyte(0);
}

unsigned char cmd_read_eeprom(int address)
{
        spibyte(0xa0);
        spibyte((address>>8)&0xff);
        spibyte(address&0xff);
        return spibyte(0);
}

unsigned char cmd_read_fuses()
{
	spibyte(0x50);
	spibyte(0);
	spibyte(0);
	return spibyte(0);
}

unsigned char cmd_read_fusesH()
{
	spibyte(0x58);
	spibyte(0x08);
	spibyte(0);
	return spibyte(0);
}

unsigned char cmd_read_fusesEx()
{
	spibyte(0x50);
	spibyte(0x08);
	spibyte(0);
	return spibyte(0);
}

unsigned char cmd_read_lock()
{
	spibyte(0x58);
	spibyte(0x0);
	spibyte(0);
	return spibyte(0);
}

unsigned char cmd_read_cal(unsigned char addr)
{
	spibyte(0x38);
	spibyte(0x0);
	spibyte(addr);
	return spibyte(0);
}


/*---------------------------------------------------------*/


void cmd_chip_erase()
{
        spibyte(0xac);
        spibyte(0x80);
        spibyte(0x0);
        spibyte(0x0);
}

void cmd_write_program(int address, unsigned char data)
{
        spibyte((address&1)?0x48:0x40);
        spibyte((address>>9)&3);
        spibyte((address>>1)&0xff);
        spibyte(data);
}

void cmd_write_page(int page)
{
	//printf("page 0x%04x\n",page);
	spibyte(0x4c);
	spibyte((page>>8)&0xff);
	spibyte(page&0xff);
	spibyte(0x0);
}

void cmd_write_eeprom(int address, unsigned char data)
{
        spibyte(0xc0);
	spibyte((address>>8)&0xff);
        spibyte(address&0xff);
	spibyte(data);
}


void cmd_write_fuses(unsigned char data)
{
	spibyte(0xac);
	spibyte(0xa0);
	spibyte(0);
	spibyte(data);
}

void cmd_write_fusesH(unsigned char data)
{
	spibyte(0xac);
	spibyte(0xa8);
	spibyte(0);
	spibyte(data);
}

void cmd_write_fusesEx(unsigned char data)
{
	spibyte(0xac);
	spibyte(0xa4);
	spibyte(0);
	spibyte(data);
}

void cmd_write_lock(unsigned char data)
{
	spibyte(0xac);
	spibyte(0xe0);
	spibyte(0);
	spibyte(data|0xc0);
}



char cmd_poll_rdy()
{
	spibyte(0xf0);
	spibyte(0x00);
	spibyte(0x00);
	return spibyte(0x00)&1;
}
