#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include "common.h"

unsigned int lpu=0;	// loops per microsecond

#ifdef linux
 
void active_delay(int n)
{
    volatile int i;
    for(i=n;i;i--);
}

// Delay of n microseconds
void udelay(int n)
{
    int i;
    for (;n;n--) active_delay(lpu);
}

void delay_calibrate()
{
    struct timeval tv;
    int i,t,tm[16],m,n,mm,ni;
    char buf[256];

    message("----- Calibrating delay ...",4);
    // Let run 16 tests (m=average value)
    for (i=m=0;i<16;i++) {
        gettimeofday(&tv, NULL); t=tv.tv_usec;
        active_delay(100000);
        gettimeofday(&tv, NULL); t=tv.tv_usec-t;
        if (t<0) t+=1000000;
        tm[i]=t;
        m+=t;
    }
    m>>=4;
    // Remove long delays (those delays are due to interrupts)
    // and recalculate average (mm)
    for (i=n=mm=ni=0;i<16;i++) {
        if(tm[i]>2*m) {ni++; continue;}
        mm+=tm[i];
        n++;
    }
    mm/=n;
    if (ni) {
        sprintf(buf,"> %d interrupt(s) detected",ni);
        message(buf,2);
    }
    t=(100000)/mm;
    sprintf(buf,"> %d loops per us interval",t);
    message(buf,0);
    lpu=t;
}

#else 

unsigned long long rdtsc()
{
asm volatile ("rdtsc");
}

// Delay of n microseconds
void udelay(int n)
{
    unsigned long long t0;
    t0=rdtsc();
    while ((rdtsc()-t0)<lpu*n);
}

void delay_calibrate()
{
    char buf[256];
    long long t0;
    t0=rdtsc();
    sleep(1);
    lpu=(rdtsc()-t0)/1000000;
    sprintf(buf,"> RDTSC: %d cycles per us (MHz)",lpu);
    message(buf,0);
}

#endif

