/************************************************************************
Loader for several Hexadecimal file formats

	- Intel Hex
	- Mostek Hex
	- Motorola S-record Hex

*************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "common.h"
#include "id.h"

extern unsigned char membuf[];
extern chip_id *chip;
extern int nbytes;

unsigned int hexdata(char *p,int nb)
{
	char buf[16];
	unsigned int i;

	buf[0]='0'; buf[1]='X';
	for (i=2;i<(nb*2+2);i++) buf[i]=*(p++);
	buf[i]=0;
	sscanf(buf,"%X",&i);
	return i;
}

int loadhex(char *fn)
{
	unsigned char buf[256],dat[256],*p,*formato=NULL;
	int i,n,tipo;
	unsigned int dirbase,dir,chk,fdir,f=0;
	FILE *fp;

	if((fp=fopen(fn,"r"))==NULL) {
		sprintf(buf,"Error : Cannot open file. File :%s",fn);
		message(buf,1);
                return;
	}

        for (i=0;i<MAXBUF;i++) membuf[i]=0xff;
        nbytes=0;
	for(;;){
		fgets(buf,256,fp);
		if (feof(fp)) break;
		switch (buf[0]) {
		case ':':	/** Formato INTEL **/
			formato="Intel Hex";
			p=buf+1;
			chk=0;
			n=hexdata(p,1); p+=2;	// N de bytes de datos
			chk+=n;
			dirbase=hexdata(p,2); p+=4;
			chk+=dirbase>>8;
			chk+=dirbase&0xff;
			tipo=hexdata(p,1); p+=2;
			chk+=tipo;
			if (tipo) {chk=n=0; break;}
			for (i=0;i<n;i++) {
				dat[i]=hexdata(p,1);
				p+=2;
				chk+=dat[i];
			}
			chk+=hexdata(p,1);
			chk&=0xff;
			
			
			
			//printf("INTEL N=0x%x BASE=0x%x TIPO=%d CHK=%x\n",n,dirbase,tipo,chk);
			break;

		case ';':	/** Formato MOSTEK **/
			formato="Mostek Hex";
			p=buf+1;
			chk=0;
			n=hexdata(p,1); p+=2;
			chk+=n;
			if (n==0) break;
			dirbase=hexdata(p,2); p+=4;
			chk+=dirbase>>8;
			chk+=dirbase&0xff;
			for (i=0;i<n;i++) {
				dat[i]=hexdata(p,1);
				p+=2;
				chk+=dat[i];
			}
			chk-=hexdata(p,2);
			//printf("MOS N=0x%x BASE=0x%x CHK=%x\n",n,dirbase,chk);
			break;

		case 'S':	/** Formato MOTOROLA **/
			formato="Motorola S-record";
			n=chk=0;
			p=buf+1;
			tipo=*(p++)-'0';
			if (tipo==0) break;
			if (tipo>3) break;
			n=hexdata(p,1); p+=2;
			chk+=n;
			n-=tipo+2;
			dirbase=hexdata(p,tipo+1); p+=(tipo+1)*2;
			for (i=0;i<tipo+1;i++) chk+=(dirbase>>(i*8))&0xff;
			for (i=0;i<n;i++) {
				dat[i]=hexdata(p,1);
				p+=2;
				chk+=dat[i];
			}
			chk+=hexdata(p,2);
			chk++;
			chk&=0xff;
			//printf("MOTOROLA TIPO=%d N=0x%x BASE=0x%x CHK=%x\n",tipo,n,dirbase,chk);
			break;

		default:
			//sprintf(buf,"Error : Unrecognized File Format. File :%s",fn);
                        //message(buf,1);
                        fclose(fp); return 1;
		}
		if (n==0) continue;
		if (chk) {
			sprintf(buf,"Error : Checksum Test Failed. File:%s",fn);
                        message(buf,1); fclose(fp); return 0;
		}
		for(i=0;i<n;i++)
			if ((dirbase+i)<MAXBUF) membuf[dirbase+i]=dat[i];
                        else {
                            message("Address out of range",1);
                            fclose(fp);
                            return;
                        }
		if (dirbase+n>nbytes) nbytes=dirbase+n;
		if (f==0) fdir=dirbase;
		f=1;
	}
        if (!formato) return 1;
	fclose(fp);
	sprintf(buf,"----- File Loaded :%s",fn);
        message(buf,4);
        sprintf(buf,"> Format :%s, Base :0x%X, Size :%d",formato,fdir,nbytes-fdir);
        message(buf,0);
        return 0;
}

