#include <stdlib.h>
#include <stdio.h>
#include "delay.h"
#include "bbang.h"
#include "common.h"
#include "id.h"

extern int ckdelay;
extern int nbytes;
extern unsigned char membuf[];
extern chip_id idt[],*chip;

int identify_device()
{
    int i,id;
    char buf[256];

    message("----- Identifying Device...",4);
    id=cmd_read_signature(0); id<<=8;
    id|=cmd_read_signature(1); id<<=8;
    id|=cmd_read_signature(2);

    sprintf(buf,"<> Device_ID = 0x%06X",id);
    message(buf,0);
    if (id==0x000102) message("<> The device is locked",2);
    for (i=1;idt[i].id;i++) {
        if (id==idt[i].id) {
             chip=&idt[i];
             sprintf(buf,"<> device found: %s",chip->name);
             message(buf,0);
             show_chip();
             return 0;
        }
    }
    message("<> Unknown device",1);
    return -1;
}

int read_flash()
{
    int i;
    for (i=0;i<chip->flash_size;i++) {
        if ((i&127)==0) refresh_percent(i*100/chip->flash_size);
        membuf[i]=cmd_read_program(i);
    }
    nbytes=chip->flash_size;
    refresh_percent(100);
    return 0;
}

int check_flash()
{
    int i;
    for (i=0;i<chip->flash_size;i++) {
        if ((i&127)==0) refresh_percent(i*100/chip->flash_size);
        if (cmd_read_program(i)!=0xff) return 1;
    }
    refresh_percent(100);
    return 0;
}

int verify_flash()
{
    int i;
    for (i=0;i<nbytes;i++) {
        if ((i&127)==0) refresh_percent(i*100/nbytes);
        if (cmd_read_program(i)!=membuf[i]) return 1;
    }
    refresh_percent(100);
    return 0;
}

int read_eeprom()
{
    int i;
    for (i=0;i<chip->eeprom_size;i++) membuf[i]=cmd_read_eeprom(i);
    nbytes=chip->eeprom_size;
    refresh_percent(100);
    return 0;
}

int write_flash_normal()
{
    int i,j,n,pmask,psh,page,cpage;
    unsigned char d;
    char buf[256];

    n=nbytes;
    if (n>chip->flash_size) {
         n=chip->flash_size;
         sprintf(buf,"<> Warning: Only %d (0x%X) bytes written",n,n);
         message(buf,2);
    }
    for (i=0;i<n;i++) {
        if ((i&15)==0) refresh_percent(i*100/n);
        d=cmd_read_program(i);
        if (d==membuf[i]) continue;
        if ((membuf[i]&d)!=membuf[i]) {
            sprintf(buf,"***** Error addr=0x%04X data=0x%02X memory=0x02X",
                    i,membuf[i],d);
            message(buf,1);
            message("***** (Chip Erase is required)",1);
            return 1;
        }
	cmd_write_program(i, membuf[i]);
	if (membuf[i]==chip->p0) udelay(TPROG);
        else {
                for (j=0;j<NTOUT;j++) if (cmd_read_program(i)==membuf[i]) break;
                if (j==NTOUT) {
                     sprintf(buf,"***** Error addr=0x%04X data=0x%02X",
                             i,membuf[i]);
                     message(buf,1);
                     message("***** TIMEOUT",1);
                     return 1;
                }
        }
    }
    refresh_percent(100);
    return 0;
}

int write_flash_paged()
{
    int i,j,k,n,np;
    unsigned char d;
    char buf[256];

    n=nbytes;
    if (n>chip->flash_size) {
         n=chip->flash_size;
         sprintf(buf,"<> Warning: Only %d (0x%X) bytes written",n,n);
         message(buf,2);
    }
    
#define  bpp  (2*chip->page_size)
    
    np=n/bpp; if (n%bpp) np++; // number of pages

    for (k=0;k<np;k++) {
        // printf("PAGE: 0x%x\n",k);
        refresh_percent(k*100/np);
	// checking if programming is needed/possible
	for (i=k*bpp;i<k*bpp+bpp;i++) {
            d=cmd_read_program(i);
	    if (d!=membuf[i]) break;
	}
	if (i==k*bpp+bpp) continue;
        else if ((membuf[i]&d)!=membuf[i]) {
            sprintf(buf,"***** Error addr=0x%04X data=0x%02X memory=0x02X",
                    i,membuf[i],d);
            message(buf,1);
            message("***** (Chip Erase is required)",1);
            return 1;
        }
	// filling page before programming
	for (i=0;i<bpp;i++) cmd_write_program(i, membuf[k*bpp+i]);
	// programming
	cmd_write_page(k*chip->page_size);
	if (chip->p0!=1) {
	  // Old polling
	  i=k*bpp;
	  if (membuf[i]==chip->p0) udelay(TPROG);
          else {
                for (j=0;j<NTOUT;j++) if (cmd_read_program(i)==membuf[i]) break;
                if (j==NTOUT) {
                     sprintf(buf,"***** Error Page_addr=0x%04X data=0x%02X",
                             i,membuf[i]);
                     message(buf,1);
                     message("***** TIMEOUT",1);
                     return 1;
                }
          }
	} else {
	  // Poll_RDY command
	  for (j=0;j<NTOUT;j++) if(!cmd_poll_rdy()) break;
	  if (j==NTOUT) {
	  	message("***** TIMEOUT",1); return 1;
	  }
	  //printf("Polling OK\n");
	}
    }

#undef bpp
    
    refresh_percent(100);
    return 0;
}

int write_flash()
{
    if (chip->page_size) write_flash_paged();
    else write_flash_normal();
}

int write_eeprom()
{
    int i,j,n;
    unsigned char d;
    char buf[256];

    n=nbytes;
    if (n>chip->eeprom_size) {
         n=chip->eeprom_size;
         sprintf(buf,"<> Warning: Only %d (0x%X) bytes written",n,n);
         message(buf,2);
    }
    for (i=0;i<n;i++) {
        if ((i&15)==0) refresh_percent(i*100/n);
        d=cmd_read_eeprom(i);
        if (d==membuf[i]) continue;

        cmd_write_eeprom(i, membuf[i]);
	if (chip->p1!=1) {
          // Old polling
	  if (membuf[i]==chip->p1 || membuf[i]==chip->p2)
            udelay(TPROG);
          else {
            for (j=0;j<NTOUT;j++) if (cmd_read_eeprom(i)==membuf[i]) break;
            if (j==NTOUT) {
                 sprintf(buf,"***** Error addr=0x%04X data=0x%02X",
                         i,membuf[i]);
                 message(buf,1);
                 message("***** TIMEOUT",1);
                 return 1;
            }
          }
	} else {
	  for (j=0;j<NTOUT;j++) if (!cmd_poll_rdy()) break;
	  if (j==NTOUT) {message("***** TIMEOUT",1); return 1;}
	  //printf("Polling OK\n");
	}
    }
    refresh_percent(100);
    return 0;
}

int chip_erase()
{
	int j;
        cmd_chip_erase();
	if (chip->p0==1) {
	  for (j=0;j<NTOUT;j++) if (!cmd_poll_rdy()) break;
	  if (j==NTOUT) {message("***** TIMEOUT",1); return 1;}
	  //printf("Polling erase OK\n");  
	} else usleep(TERASE);
	return 0;
}
