#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "common.h"
#include "ioport.h"

#ifndef linux
#include <sys/perm.h>
#endif

#define iobase 0x378

#define DATA (iobase)
#define STAT (iobase+1)

unsigned char datar;

struct {
    unsigned char vcc,invvcc;		// Outputs to (DATA)
    unsigned char reset,invreset;
    unsigned char sck,invsck;
    unsigned char mosi,invmosi;
    unsigned char miso,invmiso;		// Input from (STAT)
} parspi;

// Superprog Interface pins
#define VCC 2
#define INVVCC 0

#define VPP 3
#define INVVPP 1

#define RESET 4
#define INVRESET 1

#define SCK 5
#define INVSCK 1

#define MOSI 6
#define INVMOSI 1

#define MISO 7		// Bit Busy
#define INVMISO 0


#ifndef PARCONF
#define PARCONF "parallel.cfg"
#endif

void set_Vcc_H() // and Or RESET low
{
	datar|=parspi.vcc;	datar^=parspi.invvcc;
	datar&=~parspi.reset; 	datar^=parspi.invreset;
	
        outb(datar,DATA);
}

void set_Vcc_L() // Or RESET High
{
	datar&=~parspi.vcc;	datar^=parspi.invvcc;;
	datar|=parspi.reset; 	datar^=parspi.invreset;

        outb(datar,DATA);
}

void set_SCK_H()
{
        datar|=parspi.sck;	datar^=parspi.invsck;
        outb(datar,DATA);
}
void set_SCK_L()
{
        datar&=~parspi.sck;	datar^=parspi.invsck;
        outb(datar,DATA);
}
void set_MOSI_H()
{
        datar|=parspi.mosi;	datar^=parspi.invmosi;
        outb(datar,DATA);
}
void set_MOSI_L()
{
        datar&=~parspi.mosi;	datar^=parspi.invmosi;
        outb(datar,DATA);
}

int get_MISO()
{
        return (inb(STAT)&parspi.miso)^parspi.invmiso;
}


void init_io()
{
    int i;
    FILE *fp;
    char *p,buf[256];
    static const char *pname[5]={"VCC","RESET","SCK","MISO","MOSI"};
    static unsigned char *pvar[5]=
    	{&parspi.vcc,&parspi.reset,&parspi.sck,&parspi.miso,&parspi.mosi};
    static unsigned char *pvari[5]=
    	{&parspi.invvcc,&parspi.invreset,&parspi.invsck,
    	 &parspi.invmiso,&parspi.invmosi};			

    // Default Pin Configuration

    parspi.vcc=(1<<VCC);	parspi.invvcc=(INVVCC<<VCC);
    parspi.reset=(1<<RESET);	parspi.invreset=(INVRESET<<RESET);
    parspi.sck=(1<<SCK);	parspi.invsck=(INVSCK<<SCK);
    parspi.mosi=(1<<MOSI);	parspi.invmosi=(INVMOSI<<MOSI);
    parspi.miso=(1<<MISO);	parspi.invmiso=(INVMISO<<MISO);

    // Get pin configuration from file
    
    if ((fp=fopen(PARCONF,"r"))!=NULL) {
        sprintf(buf,"----- Reading Parallel Port Configuration (%s)...",PARCONF);
        message(buf,4);
        do {
	    fgets(buf,254,fp);
	    for (i=0;i<5;i++) {
	      if ((p=strstr(buf,pname[i]))!=NULL) {
		while (*p!=' ' && *p!='\t') p++;
		*pvar[i]=(1<<atoi(p));
		while (*p==' ' || *p=='\t') p++;
		while (*p!=' ' && *p!='\t') p++;
		*pvari[i]=(atoi(p))?*pvar[i]:0;
		sprintf (buf,"%s:\t0x%02X %s",pname[i],*pvar[i],
			(*pvari[i])?"Inv.":"");
		message(buf,4);
	      }
	    }
	} while (!feof(fp));
    }

    sprintf(buf,"----- Opening Parallel Port...");
    message(buf,4);

    //iobase=0;
#ifdef linux
    if (iopl(3)) {message("ERROR: root privileges required",1); return;}
#else
    if (iopl(3)) {message("ERROR: Execute ioperm -i",1); return;}
#endif    
    sprintf(buf,"----- IO Base: 0x%03X",iobase);
    message(buf,4);
    set_Vcc_L();
    set_SCK_L();
    set_MOSI_L();
    //message("----- Parallel port OK",4);
}
